@@ -4,9 +4,12 @@ package cni
 
 import (
 	"fmt"
-	"net"
+	"io/ioutil"
+	"os"
 	"os/exec"
+	"strconv"
 	"strings"
+	"time"
 
 	"github.com/sirupsen/logrus"
 
@@ -15,6 +18,8 @@ import (
 	"github.com/containernetworking/plugins/pkg/ip"
 	"github.com/containernetworking/plugins/pkg/ns"
 	"github.com/vishvananda/netlink"
+
+	"k8s.io/apimachinery/pkg/util/wait"
 )
 
 func renameLink(curName, newName string) error {
@@ -36,6 +41,10 @@ func renameLink(curName, newName string) error {
 	return nil
 }
 
+func setSysctl(sysctl string, newVal int) error {
+	return ioutil.WriteFile(sysctl, []byte(strconv.Itoa(newVal)), 0640)
+}
+
 func moveIfToNetns(ifname string, netns ns.NetNS) error {
 	vfDev, err := netlink.LinkByName(ifname)
 	if err != nil {
@@ -50,39 +59,48 @@ func moveIfToNetns(ifname string, netns ns.NetNS) error {
 	return nil
 }
 
-func setupNetwork(link netlink.Link, macAddress, ipAddress, gatewayIP string) error {
-	hwAddr, err := net.ParseMAC(macAddress)
-	if err != nil {
-		return fmt.Errorf("failed to parse mac address for %s: %v", link.Attrs().Name, err)
+func setupNetwork(link netlink.Link, ifInfo *PodInterfaceInfo) error {
+	if err := netlink.LinkSetHardwareAddr(link, ifInfo.MAC); err != nil {
+		return fmt.Errorf("failed to add mac address %s to %s: %v", ifInfo.MAC, link.Attrs().Name, err)
 	}
-	err = netlink.LinkSetHardwareAddr(link, hwAddr)
-	if err != nil {
-		return fmt.Errorf("failed to add mac address %s to %s: %v", macAddress, link.Attrs().Name, err)
-	}
-	addr, err := netlink.ParseAddr(ipAddress)
-	if err != nil {
-		return err
-	}
-	err = netlink.AddrAdd(link, addr)
-	if err != nil {
-		return fmt.Errorf("failed to add IP addr %s to %s: %v", ipAddress, link.Attrs().Name, err)
+	addr := &netlink.Addr{IPNet: ifInfo.IP}
+	if err := netlink.AddrAdd(link, addr); err != nil {
+		return fmt.Errorf("failed to add IP addr %s to %s: %v", ifInfo.IP, link.Attrs().Name, err)
 	}
 
-	gw := net.ParseIP(gatewayIP)
-	if gw == nil {
-		return fmt.Errorf("parse ip of gateway failed")
+	var foundDefault bool
+	for _, route := range ifInfo.Routes {
+		if err := ip.AddRoute(route.Dest, route.NextHop, link); err != nil {
+			return fmt.Errorf("failed to add pod route %v via %v: %v", route.Dest, route.NextHop, err)
+		}
+
+		if ones, _ := route.Dest.Mask.Size(); ones == 0 {
+			foundDefault = true
+		}
 	}
-	return ip.AddRoute(nil, gw, link)
+
+	if !foundDefault {
+		// If the pod routes did not include a default route,
+		// add a "default" default route via the pod's gateway, if
+		// one exists
+		if ifInfo.GW != nil {
+			if err := ip.AddRoute(nil, ifInfo.GW, link); err != nil {
+				return fmt.Errorf("failed to add gateway route: %v", err)
+			}
+		}
+	}
+
+	return nil
 }
 
-func setupInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddress, gatewayIP string, mtu int) (*current.Interface, *current.Interface, error) {
+func setupInterface(netns ns.NetNS, containerID, ifName string, ifInfo *PodInterfaceInfo) (*current.Interface, *current.Interface, error) {
 	hostIface := &current.Interface{}
 	contIface := &current.Interface{}
 
 	var oldHostVethName string
 	err := netns.Do(func(hostNS ns.NetNS) error {
 		// create the veth pair in the container and move host end into host netns
-		hostVeth, containerVeth, err := ip.SetupVeth(ifName, mtu, hostNS)
+		hostVeth, containerVeth, err := ip.SetupVeth(ifName, ifInfo.MTU, hostNS)
 		if err != nil {
 			return err
 		}
@@ -94,11 +112,11 @@ func setupInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddress,
 			return fmt.Errorf("failed to lookup %s: %v", contIface.Name, err)
 		}
 
-		err = setupNetwork(link, macAddress, ipAddress, gatewayIP)
+		err = setupNetwork(link, ifInfo)
 		if err != nil {
 			return err
 		}
-		contIface.Mac = macAddress
+		contIface.Mac = ifInfo.MAC.String()
 		contIface.Sandbox = netns.Path()
 
 		oldHostVethName = hostVeth.Name
@@ -119,7 +137,7 @@ func setupInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddress,
 }
 
 // Setup sriov interface in the pod
-func setupSriovInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddress, gatewayIP string, mtu int, pciAddrs string) (*current.Interface, *current.Interface, error) {
+func setupSriovInterface(netns ns.NetNS, containerID, ifName string, ifInfo *PodInterfaceInfo, pciAddrs string) (*current.Interface, *current.Interface, error) {
 	hostIface := &current.Interface{}
 	contIface := &current.Interface{}
 
@@ -167,11 +185,37 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddr
 	hostIface.Mac = link.Attrs().HardwareAddr.String()
 
 	// 6. set MTU on VF representor
-	if err = netlink.LinkSetMTU(link, mtu); err != nil {
+	if err = netlink.LinkSetMTU(link, ifInfo.MTU); err != nil {
 		return nil, nil, fmt.Errorf("failed to set MTU on %s: %v", hostIface.Name, err)
 	}
 
-	// 7. Move VF to Container namespace
+	physlink, err := netlink.LinkByName(uplink)
+	if err != nil {
+		return nil, nil, fmt.Errorf("failed getting link for %s: %v", uplink, err)
+	}
+
+	// 7. Clean up any left over bandwidth configuration, before proceeding.
+	// Probably put this in a function that cleans up/resets other configs too.
+	err = netlink.LinkSetVfTxRate(physlink, vfIndex, 0)
+	if err != nil {
+		logrus.Infof("failed to clean up bandwidth on %s (%s/%d): %v", pciAddrs, physlink, vfIndex, err)
+	}
+
+	// 8. set rate, if any.
+	if ifInfo.Egress > 0 {
+		// Rate in Mbps, LinkSetVfTxRate takes an int, and a value of 0 means no rate limit.
+		if ifInfo.Egress < 1000000 {
+			return nil, nil, fmt.Errorf("rate of %d bps not supported on device %s/%d", ifInfo.Egress, uplink, vfIndex)
+		}
+		// in Mbps
+		egressMbps := int(ifInfo.Egress / 1000000)
+		err = netlink.LinkSetVfTxRate(physlink, vfIndex, egressMbps)
+		if err != nil {
+			return nil, nil, fmt.Errorf("failed setting link bandwidth of %d Mbps on device %s/%d: %v", egressMbps, uplink, vfIndex, err)
+		}
+	}
+
+	// 9. Move VF to Container namespace
 	err = moveIfToNetns(vfNetdevice, netns)
 	if err != nil {
 		return nil, nil, err
@@ -187,7 +231,7 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddr
 		if err != nil {
 			return err
 		}
-		err = netlink.LinkSetMTU(link, mtu)
+		err = netlink.LinkSetMTU(link, ifInfo.MTU)
 		if err != nil {
 			return err
 		}
@@ -196,12 +240,12 @@ func setupSriovInterface(netns ns.NetNS, containerID, ifName, macAddress, ipAddr
 			return err
 		}
 
-		err = setupNetwork(link, macAddress, ipAddress, gatewayIP)
+		err = setupNetwork(link, ifInfo)
 		if err != nil {
 			return err
 		}
 
-		contIface.Mac = macAddress
+		contIface.Mac = ifInfo.MAC.String()
 		contIface.Sandbox = netns.Path()
 
 		return nil
@@ -228,7 +272,7 @@ var iptablesCommands = [][]string{
 }
 
 // ConfigureInterface sets up the container interface
-func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAddress string, ipAddress string, gatewayIP string, mtu int, ingress, egress int64) ([]*current.Interface, error) {
+func (pr *PodRequest) ConfigureInterface(namespace string, podName string, ifInfo *PodInterfaceInfo) ([]*current.Interface, error) {
 	netns, err := ns.GetNS(pr.Netns)
 	if err != nil {
 		return nil, fmt.Errorf("failed to open netns %q: %v", pr.Netns, err)
@@ -240,11 +284,11 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAd
 	logrus.Debugf("CNI Conf %v", pr.CNIConf)
 	if pr.CNIConf.DeviceID != "" {
 		// SR-IOV Case
-		hostIface, contIface, err = setupSriovInterface(netns, pr.SandboxID, pr.IfName, macAddress, ipAddress, gatewayIP, mtu, pr.CNIConf.DeviceID)
+		hostIface, contIface, err = setupSriovInterface(netns, pr.SandboxID, pr.IfName, ifInfo, pr.CNIConf.DeviceID)
 
 	} else {
 		// General case
-		hostIface, contIface, err = setupInterface(netns, pr.SandboxID, pr.IfName, macAddress, ipAddress, gatewayIP, mtu)
+		hostIface, contIface, err = setupInterface(netns, pr.SandboxID, pr.IfName, ifInfo)
 	}
 	if err != nil {
 		return nil, err
@@ -252,12 +296,23 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAd
 
 	ifaceID := fmt.Sprintf("%s_%s", namespace, podName)
 
+	// Find and remove any existing OVS port with this iface-id. Pods can
+	// have multiple sandboxes if some are waiting for garbage collection,
+	// but only the latest one should have the iface-id set.
+	uuids, _ := ovsFind("Interface", "_uuid", "external-ids:iface-id="+ifaceID)
+	for _, uuid := range uuids {
+		if out, err := ovsExec("remove", "Interface", uuid, "external-ids", "iface-id"); err != nil {
+			logrus.Warningf("failed to clear stale OVS port %q iface-id %q: %v\n  %q", uuid, ifaceID, err, out)
+		}
+	}
+
+	// Add the new sandbox's OVS port
 	ovsArgs := []string{
 		"add-port", "br-int", hostIface.Name, "--", "set",
 		"interface", hostIface.Name,
-		fmt.Sprintf("external_ids:attached_mac=%s", macAddress),
+		fmt.Sprintf("external_ids:attached_mac=%s", ifInfo.MAC),
 		fmt.Sprintf("external_ids:iface-id=%s", ifaceID),
-		fmt.Sprintf("external_ids:ip_address=%s", ipAddress),
+		fmt.Sprintf("external_ids:ip_address=%s", ifInfo.IP),
 		fmt.Sprintf("external_ids:sandbox=%s", pr.SandboxID),
 	}
 
@@ -269,7 +324,7 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAd
 		return nil, err
 	}
 
-	if ingress > 0 || egress > 0 {
+	if ifInfo.Ingress > 0 || ifInfo.Egress > 0 {
 		var l netlink.Link
 		l, err = netlink.LinkByName(hostIface.Name)
 		if err != nil {
@@ -280,7 +335,7 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAd
 			return nil, fmt.Errorf("failed to set host veth txqlen: %v", err)
 		}
 
-		if err := setPodBandwidth(pr.SandboxID, hostIface.Name, ingress, egress); err != nil {
+		if err := setPodBandwidth(pr.SandboxID, hostIface.Name, ifInfo.Ingress, ifInfo.Egress); err != nil {
 			return nil, err
 		}
 	}
@@ -293,10 +348,28 @@ func (pr *PodRequest) ConfigureInterface(namespace string, podName string, macAd
 				return fmt.Errorf("could not set up pod iptables rules: %s", string(out))
 			}
 		}
-		return nil
+
+		if _, err := os.Stat("/proc/sys/net/ipv6/conf/all/dad_transmits"); !os.IsNotExist(err) {
+			err = setSysctl("/proc/sys/net/ipv6/conf/all/dad_transmits", 0)
+			if err != nil {
+				logrus.Warningf("failed to disable IPv6 DAD: %q", err)
+			}
+		}
+		return ip.SettleAddresses(contIface.Name, 10)
 	})
 	if err != nil {
-		return nil, err
+		logrus.Warningf("failed to configure container network namespace: %q", err)
+	}
+
+	err = wait.PollImmediate(100*time.Millisecond, 20*time.Second, func() (bool, error) {
+		stdout, err := ofctlExec("dump-flows", "br-int")
+		if err != nil {
+			return false, nil
+		}
+		return strings.Contains(stdout, ifInfo.IP.IP.String()), nil
+	})
+	if err != nil {
+		return nil, fmt.Errorf("timed out dumping br-int flow entries for sandbox: %v", err)
 	}
 
 	return []*current.Interface{hostIface, contIface}, nil
