@@ -1,13 +1,13 @@
 package ovn
 
 import (
-	"encoding/json"
 	"fmt"
-	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
-	"github.com/sirupsen/logrus"
 	"hash/fnv"
 	"strconv"
 	"strings"
+
+	util "github.com/ovn-org/ovn-kubernetes/go-controller/pkg/util"
+	"github.com/sirupsen/logrus"
 )
 
 // hash the provided input to make it a valid addressSet or portGroup name.
@@ -60,7 +60,7 @@ func (oc *Controller) forEachAddressSetUnhashedName(iteratorFn func(
 	return nil
 }
 
-func (oc *Controller) setAddressSet(hashName string, addresses []string) {
+func setAddressSet(hashName string, addresses []string) {
 	logrus.Debugf("setAddressSet for %s with %s", hashName, addresses)
 	if len(addresses) == 0 {
 		_, stderr, err := util.RunOVNNbctl("clear", "address_set",
@@ -72,7 +72,7 @@ func (oc *Controller) setAddressSet(hashName string, addresses []string) {
 		return
 	}
 
-	ips := strings.Join(addresses, " ")
+	ips := `"` + strings.Join(addresses, `" "`) + `"`
 	_, stderr, err := util.RunOVNNbctl("set", "address_set",
 		hashName, fmt.Sprintf("addresses=%s", ips))
 	if err != nil {
@@ -81,7 +81,7 @@ func (oc *Controller) setAddressSet(hashName string, addresses []string) {
 	}
 }
 
-func (oc *Controller) createAddressSet(name string, hashName string,
+func createAddressSet(name string, hashName string,
 	addresses []string) {
 	logrus.Debugf("createAddressSet with %s and %s", name, addresses)
 	addressSet, stderr, err := util.RunOVNNbctl("--data=bare",
@@ -104,7 +104,7 @@ func (oc *Controller) createAddressSet(name string, hashName string,
 		return
 	}
 
-	ips := strings.Join(addresses, " ")
+	ips := `"` + strings.Join(addresses, `" "`) + `"`
 
 	// An addressSet has already been created. Just set addresses.
 	if addressSet != "" {
@@ -135,7 +135,7 @@ func (oc *Controller) createAddressSet(name string, hashName string,
 	}
 }
 
-func (oc *Controller) deleteAddressSet(hashName string) {
+func deleteAddressSet(hashName string) {
 	logrus.Debugf("deleteAddressSet %s", hashName)
 
 	_, stderr, err := util.RunOVNNbctl("--if-exists", "destroy",
@@ -147,8 +147,7 @@ func (oc *Controller) deleteAddressSet(hashName string) {
 	}
 }
 
-func (oc *Controller) createPortGroup(name string,
-	hashName string) (string, error) {
+func createPortGroup(name string, hashName string) (string, error) {
 	logrus.Debugf("createPortGroup with %s", name)
 	portGroup, stderr, err := util.RunOVNNbctl("--data=bare",
 		"--no-heading", "--columns=_uuid", "find", "port_group",
@@ -173,7 +172,7 @@ func (oc *Controller) createPortGroup(name string,
 	return portGroup, nil
 }
 
-func (oc *Controller) deletePortGroup(hashName string) {
+func deletePortGroup(hashName string) {
 	logrus.Debugf("deletePortGroup %s", hashName)
 
 	portGroup, stderr, err := util.RunOVNNbctl("--data=bare",
@@ -198,28 +197,6 @@ func (oc *Controller) deletePortGroup(hashName string) {
 	}
 }
 
-func (oc *Controller) getIPFromOvnAnnotation(ovnAnnotation string) string {
-	if ovnAnnotation == "" {
-		return ""
-	}
-
-	var ovnAnnotationMap map[string]string
-	err := json.Unmarshal([]byte(ovnAnnotation), &ovnAnnotationMap)
-	if err != nil {
-		logrus.Errorf("Error in json unmarshaling ovn annotation "+
-			"(%v)", err)
-		return ""
-	}
-
-	ipAddressMask := strings.Split(ovnAnnotationMap["ip_address"], "/")
-	if len(ipAddressMask) != 2 {
-		logrus.Errorf("Error in splitting ip address")
-		return ""
-	}
-
-	return ipAddressMask[0]
-}
-
 func stringSliceMembership(slice []string, key string) bool {
 	for _, val := range slice {
 		if val == key {
